#!/usr/bin/env python
# Run tests/ansible/all.yml under Ansible and Ansible-Mitogen

import collections
import glob
import os
import signal
import sys

import jinja2

import ci_lib


TEMPLATES_DIR = os.path.join(ci_lib.GIT_ROOT, 'tests/ansible/templates')
TESTS_DIR = os.path.join(ci_lib.GIT_ROOT, 'tests/ansible')
HOSTS_DIR = os.path.join(ci_lib.TMP, 'hosts')


def pause_if_interactive():
    if os.path.exists('/tmp/interactive'):
        while True:
            signal.pause()


interesting = ci_lib.get_interesting_procs()


with ci_lib.Fold('unit_tests'):
    os.environ['SKIP_MITOGEN'] = '1'
    ci_lib.run('./run_tests -v')


ci_lib.check_stray_processes(interesting)


with ci_lib.Fold('docker_setup'):
    containers = ci_lib.container_specs(ci_lib.DISTRO_SPECS.split())
    ci_lib.start_containers(containers)


with ci_lib.Fold('job_setup'):
    os.chdir(TESTS_DIR)
    os.chmod('../data/docker/mitogen__has_sudo_pubkey.key', int('0600', 8))

    ci_lib.run("mkdir %s", HOSTS_DIR)
    for path in glob.glob(TESTS_DIR + '/hosts/*'):
        if not path.endswith('default.hosts'):
            ci_lib.run("ln -s %s %s", path, HOSTS_DIR)

    distros = collections.defaultdict(list)
    families = collections.defaultdict(list)
    for container in containers:
        distros[container['distro']].append(container['name'])
        families[container['family']].append(container['name'])

    jinja_env = jinja2.Environment(
        loader=jinja2.FileSystemLoader(searchpath=TEMPLATES_DIR),
        lstrip_blocks=True,  # Remove spaces and tabs from before a block
        trim_blocks=True,  # Remove first newline after a block
    )
    inventory_template = jinja_env.get_template('test-targets.j2')
    inventory_path = os.path.join(HOSTS_DIR, 'target')

    with open(inventory_path, 'w') as fp:
        fp.write(inventory_template.render(
            containers=containers,
            distros=distros,
            families=families,
        ))

    ci_lib.dump_file(inventory_path)

    if not ci_lib.exists_in_path('sshpass'):
        ci_lib.run("sudo apt-get update")
        ci_lib.run("sudo apt-get install -y sshpass")


with ci_lib.Fold('ansible'):
    playbook = os.environ.get('PLAYBOOK', 'all.yml')
    try:
        ci_lib.run('./run_ansible_playbook.py %s -i "%s" %s',
            playbook, HOSTS_DIR, ' '.join(sys.argv[1:]))
    except:
        pause_if_interactive()
        raise


ci_lib.check_stray_processes(interesting, containers)

pause_if_interactive()



import os
import unittest

import mitogen.core

import testlib
import simple_pkg.ping


# TODO: this is a joke. 2/3 interop is one of the hardest bits to get right.
# There should be 100 tests in this file.

@unittest.skipIf(
    os.uname()[0] == 'Darwin' and int(os.uname()[2].partition('.')[0]) >= 21,
    "Python 2.x not shipped on macOS 12.3+ (Darwin 21.4+, Monterey)",
)
class TwoThreeCompatTest(testlib.RouterMixin, testlib.TestCase):
    if mitogen.core.PY3:
        python_path = 'python2'
    else:
        python_path = 'python3'

    def test_succeeds(self):
        spare = self.router.local()
        target = self.router.local(python_path=self.python_path)

        spare2, = target.call(simple_pkg.ping.ping, spare)
        self.assertEqual(spare.context_id, spare2.context_id)
        self.assertEqual(spare.name, spare2.name)



#!/usr/bin/env python

import ci_lib

ci_lib.run_batches([
    [
        'python -m pip --no-python-version-warning --disable-pip-version-check "debops[ansible]==2.1.2"',
    ],
])

ci_lib.run('ansible-galaxy collection install debops.debops:==2.1.2')



import inspect
import json
import os
import sys
import unittest

import mitogen.master
from mitogen.core import b

import testlib


class ConstructorTest(testlib.TestCase):
    klass = mitogen.master.ModuleFinder

    def test_simple(self):
        self.klass()


class ReprTest(testlib.TestCase):
    klass = mitogen.master.ModuleFinder

    def test_simple(self):
        self.assertEqual('ModuleFinder()', repr(self.klass()))


class IsStdlibNameTest(testlib.TestCase):
    func = staticmethod(mitogen.master.is_stdlib_name)

    def test_builtin(self):
        import sys
        self.assertTrue(self.func('sys'))

    def test_stdlib_1(self):
        import logging
        self.assertTrue(self.func('logging'))

    def test_stdlib_2(self):
        # virtualenv only symlinks some paths to its local site-packages
        # directory. Ensure both halves of the search path return the correct
        # result.
        import email
        self.assertTrue(self.func('email'))

    def test_mitogen_core(self):
        import mitogen.core
        self.assertFalse(self.func('mitogen.core'))

    def test_mitogen_fakessh(self):
        import mitogen.fakessh
        self.assertFalse(self.func('mitogen.fakessh'))


class GetMainModuleDefectivePython3x(testlib.TestCase):
    klass = mitogen.master.DefectivePython3xMainMethod

    def call(self, fullname):
        return self.klass().find(fullname)

    def test_builtin(self):
        self.assertEqual(None, self.call('sys'))

    def test_not_main(self):
        self.assertEqual(None, self.call('mitogen'))

    def test_main(self):
        import __main__

        path, source, is_pkg = self.call('__main__')
        self.assertIsNotNone(path)
        self.assertTrue(os.path.exists(path))
        self.assertEqual(path, __main__.__file__)
        fp = open(path, 'rb')
        try:
            self.assertEqual(source, fp.read())
        finally:
            fp.close()
        self.assertFalse(is_pkg)


class PkgutilMethodTest(testlib.TestCase):
    klass = mitogen.master.PkgutilMethod

    def call(self, fullname):
        return self.klass().find(fullname)

    def test_empty_source_pkg(self):
        path, src, is_pkg = self.call('module_finder_testmod')
        self.assertEqual(path,
            os.path.join(testlib.MODS_DIR, 'module_finder_testmod/__init__.py'))
        self.assertEqual(mitogen.core.b(''), src)
        self.assertTrue(is_pkg)

    def test_empty_source_module(self):
        path, src, is_pkg = self.call('module_finder_testmod.empty_mod')
        self.assertEqual(path,
            os.path.join(testlib.MODS_DIR, 'module_finder_testmod/empty_mod.py'))
        self.assertEqual(mitogen.core.b(''), src)
        self.assertFalse(is_pkg)

    def test_regular_mod(self):
        from module_finder_testmod import regular_mod
        path, src, is_pkg = self.call('module_finder_testmod.regular_mod')
        self.assertEqual(path,
            os.path.join(testlib.MODS_DIR, 'module_finder_testmod/regular_mod.py'))
        self.assertEqual(mitogen.core.to_text(src),
                          inspect.getsource(regular_mod))
        self.assertFalse(is_pkg)


class SysModulesMethodTest(testlib.TestCase):
    klass = mitogen.master.SysModulesMethod

    def call(self, fullname):
        return self.klass().find(fullname)

    def test_main(self):
        import __main__
        path, src, is_pkg = self.call('__main__')
        self.assertEqual(path, __main__.__file__)

        # linecache adds a line ending to the final line if one is missing.
        with open(path, 'rb') as f:
            actual_src = f.read()
        if actual_src[-1:] != b('\n'):
            actual_src += b('\n')

        self.assertEqual(src, actual_src)
        self.assertFalse(is_pkg)

    def test_dylib_fails(self):
        # _socket comes from a .so
        import _socket
        tup = self.call('_socket')
        self.assertIsNone(tup)

    def test_builtin_fails(self):
        # sys is built-in
        tup = self.call('sys')
        self.assertIsNone(tup)


class ParentEnumerationMixin(object):
    def call(self, fullname):
        return self.klass().find(fullname)

    def test_main_fails(self):
        import __main__
        self.assertIsNone(self.call('__main__'))

    def test_dylib_fails(self):
        # _socket comes from a .so
        import _socket
        tup = self.call('_socket')
        self.assertIsNone(tup)

    def test_builtin_fails(self):
        # sys is built-in
        tup = self.call('sys')
        self.assertIsNone(tup)

    def test_plumbum_colors_like_pkg_succeeds(self):
        # plumbum has been eating too many rainbow-colored pills
        import pkg_like_plumbum.colors
        path, src, is_pkg = self.call('pkg_like_plumbum.colors')
        modpath = os.path.join(testlib.MODS_DIR, 'pkg_like_plumbum/colors.py')
        self.assertEqual(path, modpath)

        with open(modpath, 'rb') as f:
            self.assertEqual(src, f.read())
        self.assertFalse(is_pkg)

    def test_ansible_module_utils_distro_succeeds(self):
        # #590: a package that turns itself into a module.
        import pkg_like_ansible.module_utils.distro as d
        self.assertEqual(d.I_AM, "the module that replaced the package")
        self.assertEqual(
            sys.modules['pkg_like_ansible.module_utils.distro'].__name__,
            'pkg_like_ansible.module_utils.distro._distro'
        )

        # ensure we can resolve the subpackage.
        path, src, is_pkg = self.call('pkg_like_ansible.module_utils.distro')
        modpath = os.path.join(testlib.MODS_DIR,
            'pkg_like_ansible/module_utils/distro/__init__.py')
        self.assertEqual(path, modpath)
        with open(modpath, 'rb') as f:
            self.assertEqual(src, f.read())
        self.assertEqual(is_pkg, True)

        # ensure we can resolve a child of the subpackage.
        path, src, is_pkg = self.call(
            'pkg_like_ansible.module_utils.distro._distro'
        )
        modpath = os.path.join(testlib.MODS_DIR,
            'pkg_like_ansible/module_utils/distro/_distro.py')
        self.assertEqual(path, modpath)
        with open(modpath, 'rb') as f:
            self.assertEqual(src, f.read())
        self.assertEqual(is_pkg, False)

    def test_ansible_module_utils_system_distro_succeeds(self):
        # #590: a package that turns itself into a module.
        # #590: a package that turns itself into a module.
        import pkg_like_ansible.module_utils.sys_distro as d
        self.assertEqual(d.I_AM, "the system module that replaced the subpackage")
        self.assertEqual(
            sys.modules['pkg_like_ansible.module_utils.sys_distro'].__name__,
            'system_distro'
        )

        # ensure we can resolve the subpackage.
        path, src, is_pkg = self.call('pkg_like_ansible.module_utils.sys_distro')
        modpath = os.path.join(testlib.MODS_DIR,
            'pkg_like_ansible/module_utils/sys_distro/__init__.py')
        self.assertEqual(path, modpath)
        with open(modpath, 'rb') as f:
            self.assertEqual(src, f.read())
        self.assertEqual(is_pkg, True)

        # ensure we can resolve a child of the subpackage.
        path, src, is_pkg = self.call(
            'pkg_like_ansible.module_utils.sys_distro._distro'
        )
        modpath = os.path.join(testlib.MODS_DIR,
            'pkg_like_ansible/module_utils/sys_distro/_distro.py')
        self.assertEqual(path, modpath)
        with open(modpath, 'rb') as f:
            self.assertEqual(src, f.read())
        self.assertEqual(is_pkg, False)


@unittest.skipIf(sys.version_info >= (3, 4), 'Superceded in Python >= 3.4')
class ParentImpEnumerationMethodTest(ParentEnumerationMixin, testlib.TestCase):
    klass = mitogen.master.ParentImpEnumerationMethod


@unittest.skipIf(sys.version_info < (3, 4), 'Requires ModuleSpec, Python 3.4+')
class ParentSpecEnumerationMethodTest(ParentEnumerationMixin, testlib.TestCase):
    klass = mitogen.master.ParentSpecEnumerationMethod


class ResolveRelPathTest(testlib.TestCase):
    klass = mitogen.master.ModuleFinder

    def call(self, fullname, level):
        return self.klass().resolve_relpath(fullname, level)

    def test_empty(self):
        self.assertEqual('', self.call('', 0))
        self.assertEqual('', self.call('', 1))
        self.assertEqual('', self.call('', 2))

    def test_absolute(self):
        self.assertEqual('', self.call('email.utils', 0))

    def test_rel1(self):
        self.assertEqual('email.', self.call('email.utils', 1))

    def test_rel2(self):
        self.assertEqual('', self.call('email.utils', 2))

    def test_rel_overflow(self):
        self.assertEqual('', self.call('email.utils', 3))


class FakeSshTest(testlib.TestCase):
    klass = mitogen.master.ModuleFinder

    def call(self, fullname):
        return self.klass().find_related_imports(fullname)

    def test_simple(self):
        import mitogen.fakessh
        related = self.call('mitogen.fakessh')
        self.assertEqual(related, [
            'mitogen',
            'mitogen.core',
            'mitogen.parent',
        ])


class FindRelatedTest(testlib.TestCase):
    klass = mitogen.master.ModuleFinder

    def call(self, fullname):
        return self.klass().find_related(fullname)

    SIMPLE_EXPECT = set([
        u'mitogen',
        u'mitogen.core',
        u'mitogen.parent',
    ])

    if sys.version_info < (2, 7):
        SIMPLE_EXPECT.add('mitogen.compat')
        SIMPLE_EXPECT.add('mitogen.compat.tokenize')
    if sys.version_info < (2, 6):
        SIMPLE_EXPECT.add('mitogen.compat')
        SIMPLE_EXPECT.add('mitogen.compat.pkgutil')

    def test_simple(self):
        import mitogen.fakessh
        related = self.call('mitogen.fakessh')
        self.assertEqual(set(related), self.SIMPLE_EXPECT)


class DjangoMixin(object):
    WEBPROJECT_PATH = os.path.join(testlib.MODS_DIR, 'webproject')

    @classmethod
    def modules_expected_path(cls):
        if sys.version_info[0:2] < (3, 0):
            modules_expected_filename = 'modules_expected_py2x.json'
        elif sys.version_info[0:2] <= (3, 6):
            modules_expected_filename = 'modules_expected_py3x-legacy.json'
        elif sys.version_info[0:2] >= (3, 10):
            modules_expected_filename = 'modules_expected_py3x-new.json'
        return os.path.join(cls.WEBPROJECT_PATH, modules_expected_filename)

    @classmethod
    def setUpClass(cls):
        super(DjangoMixin, cls).setUpClass()
        sys.path.append(cls.WEBPROJECT_PATH)
        os.environ['DJANGO_SETTINGS_MODULE'] = 'webproject.settings'
        with open(cls.modules_expected_path(), 'rb') as f:
            cls.MODULES_EXPECTED = json.load(f)

    @classmethod
    def tearDownClass(cls):
        sys.path.remove(cls.WEBPROJECT_PATH)
        del os.environ['DJANGO_SETTINGS_MODULE']
        super(DjangoMixin, cls).tearDownClass()


class DjangoFindRelatedTest(DjangoMixin, testlib.TestCase):
    maxDiff = None

    def test_django_db(self):
        import django.db
        module_finder = mitogen.master.ModuleFinder()
        related = module_finder.find_related('django.db')
        expected = self.MODULES_EXPECTED['find_related']['django.db']
        self.assertEqual(related, expected)

    def test_django_db_models(self):
        import django.db.models
        module_finder = mitogen.master.ModuleFinder()
        related = module_finder.find_related('django.db.models')
        expected = self.MODULES_EXPECTED['find_related']['django.db.models']
        self.assertEqual(related, expected)


class DjangoFindRelatedImportsTest(DjangoMixin, testlib.TestCase):
    maxDiff = None

    def test_django_db(self):
        import django.db
        module_finder = mitogen.master.ModuleFinder()
        related = module_finder.find_related_imports('django.db')
        expected = self.MODULES_EXPECTED['find_related_imports']['django.db']
        self.assertEqual(related, expected)

    def test_django_db_models(self):
        import django.db.models
        module_finder = mitogen.master.ModuleFinder()
        related = module_finder.find_related_imports('django.db.models')
        expected = self.MODULES_EXPECTED['find_related_imports']['django.db.models']
        self.assertEqual(related, expected)



#!/usr/bin/python
# -*- coding: utf-8 -*-

# (c) 2012, Michael DeHaan <michael.dehaan@gmail.com>
# (c) 2016, Toshio Kuratomi <tkuratomi@ansible.com>
# (c) 2020, Steven Robertson <srtrumpetaggie@gmail.com>
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type

import os
import stat
import platform
import subprocess
import sys

from ansible.module_utils.basic import AnsibleModule


# trace_realpath() and _join_tracepath() adapated from stdlib posixpath.py
# https://github.com/python/cpython/blob/v3.12.6/Lib/posixpath.py#L423-L492
# Copyright (c) 2001 - 2023 Python Software Foundation
# Copyright (c) 2024 Alex Willmer <alex@moreati.org.uk>
# License: Python Software Foundation License Version 2

def trace_realpath(filename, strict=False):
    """
    Return the canonical path of the specified filename, and a trace of
    the route taken, eliminating any symbolic links encountered in the path.
    """
    path, trace, ok = _join_tracepath(filename[:0], filename, strict, seen={}, trace=[])
    return os.path.abspath(path), trace


def _join_tracepath(path, rest, strict, seen, trace):
    """
    Join two paths, normalizing and eliminating any symbolic links encountered
    in the second path.
    """
    trace.append(rest)
    if isinstance(path, bytes):
        sep = b'/'
        curdir = b'.'
        pardir = b'..'
    else:
        sep = '/'
        curdir = '.'
        pardir = '..'

    if os.path.isabs(rest):
        rest = rest[1:]
        path = sep

    while rest:
        name, _, rest = rest.partition(sep)
        if not name or name == curdir:
            # current dir
            continue
        if name == pardir:
            # parent dir
            if path:
                path, name = os.path.split(path)
                if name == pardir:
                    path = os.path.join(path, pardir, pardir)
            else:
                path = pardir
            continue
        newpath = os.path.join(path, name)
        try:
            st = os.lstat(newpath)
        except OSError:
            if strict:
                raise
            is_link = False
        else:
            is_link = stat.S_ISLNK(st.st_mode)
        if not is_link:
            path = newpath
            continue
        # Resolve the symbolic link
        if newpath in seen:
            # Already seen this path
            path = seen[newpath]
            if path is not None:
                # use cached value
                continue
            # The symlink is not resolved, so we must have a symlink loop.
            if strict:
                # Raise OSError(errno.ELOOP)
                os.stat(newpath)
            else:
                # Return already resolved part + rest of the path unchanged.
                return os.path.join(newpath, rest), trace, False
        seen[newpath] = None # not resolved symlink
        path, trace, ok = _join_tracepath(path, os.readlink(newpath), strict, seen, trace)
        if not ok:
            return os.path.join(path, rest), False
        seen[newpath] = path # resolved symlink

    return path, trace, True


def main():
    module = AnsibleModule(argument_spec=dict(
        facts_copy=dict(type=dict, default={}),
        facts_to_override=dict(type=dict, default={})
    ))

    # revert the Mitogen OSX tweak since discover_interpreter() doesn't return this info
    # NB This must be synced with mitogen.parent.Connection.get_boot_command()
    platform_release_major = int(platform.release().partition('.')[0])
    if sys.modules.get('mitogen') and sys.platform == 'darwin':
        if platform_release_major < 19 and sys.executable == '/usr/bin/python2.7':
            sys.executable = '/usr/bin/python'
        if platform_release_major in (20, 21) and sys.version_info[:2] == (2, 7):
            # only for tests to check version of running interpreter -- Mac 10.15+ changed python2
            # so it looks like it's /usr/bin/python but actually it's /System/Library/Frameworks/Python.framework/Versions/2.7/Resources/Python.app/Contents/MacOS/Python
            sys.executable = "/usr/bin/python"

    facts_copy = module.params['facts_copy']

    discovered_interpreter_python = facts_copy['discovered_interpreter_python']
    d_i_p_realpath, d_i_p_trace = trace_realpath(discovered_interpreter_python)
    d_i_p_proc = subprocess.Popen(
        [discovered_interpreter_python, '-c', 'import sys; print(sys.executable)'],
        stdout=subprocess.PIPE, stderr=subprocess.PIPE,

    )
    d_i_p_stdout, d_i_p_stderr = d_i_p_proc.communicate()

    sys_exec_realpath, sys_exec_trace = trace_realpath(sys.executable)

    result = {
        'changed': False,
        'ansible_facts': module.params['facts_to_override'],
        'discovered_and_running_samefile': os.path.samefile(
            os.path.realpath(discovered_interpreter_python),
            os.path.realpath(sys.executable),
        ),
        'discovered_python': {
            'as_seen': discovered_interpreter_python,
            'resolved': d_i_p_realpath,
            'trace': [os.path.abspath(p) for p in  d_i_p_trace],
            'sys': {
                'executable': {
                    'as_seen': d_i_p_stdout.decode('ascii').rstrip('\n'),
                    'proc': {
                        'stderr': d_i_p_stderr.decode('ascii'),
                        'returncode': d_i_p_proc.returncode,
                    },
                },
            },
        },
        'running_python': {
            'platform': {
                'release': {
                    'major': platform_release_major,
                },
            },
            'sys': {
                'executable': {
                    'as_seen': sys.executable,
                    'resolved': sys_exec_realpath,
                    'trace': [os.path.abspath(p) for p in sys_exec_trace],
                },
                'platform': sys.platform,
                'version_info': {
                    'major': sys.version_info[0],
                    'minor': sys.version_info[1],
                },
            },
        },
    }

    module.exit_json(**result)


if __name__ == '__main__':
    main()


import unittest

from ansible.utils.unsafe_proxy import AnsibleUnsafeBytes
from ansible.utils.unsafe_proxy import AnsibleUnsafeText
from ansible.utils.unsafe_proxy import wrap_var

import ansible_mitogen.utils.unsafe

import mitogen.core


class Bytes(bytes): pass
class Dict(dict): pass
class List(list): pass
class Set(set): pass
class Text(mitogen.core.UnicodeType): pass
class Tuple(tuple): pass


class CastTest(unittest.TestCase):
    def assertIsType(self, obj, cls, msg=None):
        self.assertIs(type(obj), cls, msg)

    def assertUnchanged(self, obj):
        self.assertIs(ansible_mitogen.utils.unsafe.cast(obj), obj)

    def assertCasts(self, obj, expected):
        cast = ansible_mitogen.utils.unsafe.cast
        self.assertEqual(cast(obj), expected)
        self.assertIsType(cast(obj), type(expected))

    def test_ansible_unsafe(self):
        self.assertCasts(AnsibleUnsafeBytes(b'abc'), b'abc')
        self.assertCasts(AnsibleUnsafeText(u'abc'), u'abc')

    def test_passthrough(self):
        self.assertUnchanged(0)
        self.assertUnchanged(0.0)
        self.assertUnchanged(False)
        self.assertUnchanged(True)
        self.assertUnchanged(None)
        self.assertUnchanged(b'')
        self.assertUnchanged(u'')

    def test_builtins_roundtrip(self):
        self.assertCasts(wrap_var(b''), b'')
        self.assertCasts(wrap_var({}), {})
        self.assertCasts(wrap_var([]), [])
        self.assertCasts(wrap_var(u''), u'')
        self.assertCasts(wrap_var(()), [])

    def test_subtypes_roundtrip(self):
        self.assertCasts(wrap_var(Bytes()), b'')
        self.assertCasts(wrap_var(Dict()), {})
        self.assertCasts(wrap_var(List()), [])
        self.assertCasts(wrap_var(Text()), u'')
        self.assertCasts(wrap_var(Tuple()), [])

    def test_subtype_nested_dict(self):
        obj = Dict(foo=Dict(bar=u'abc'))
        wrapped = wrap_var(obj)
        unwrapped = ansible_mitogen.utils.unsafe.cast(wrapped)
        self.assertEqual(unwrapped, {'foo': {'bar': u'abc'}})
        self.assertIsType(unwrapped, dict)
        self.assertIsType(unwrapped['foo'], dict)
        self.assertIsType(unwrapped['foo']['bar'], mitogen.core.UnicodeType)

    def test_subtype_roundtrip_list(self):
        # wrap_var() preserves sequence types, cast() does not (for now)
        obj = List([List([u'abc'])])
        wrapped = wrap_var(obj)
        unwrapped = ansible_mitogen.utils.unsafe.cast(wrapped)
        self.assertEqual(unwrapped, [[u'abc']])
        self.assertIsType(unwrapped, list)
        self.assertIsType(unwrapped[0], list)
        self.assertIsType(unwrapped[0][0], mitogen.core.UnicodeType)

    def test_subtype_roundtrip_tuple(self):
        # wrap_var() preserves sequence types, cast() does not (for now)
        obj = Tuple([Tuple([u'abc'])])
        wrapped = wrap_var(obj)
        unwrapped = ansible_mitogen.utils.unsafe.cast(wrapped)
        self.assertEqual(unwrapped, [[u'abc']])
        self.assertIsType(unwrapped, list)
        self.assertIsType(unwrapped[0], list)
        self.assertIsType(unwrapped[0][0], mitogen.core.UnicodeType)

    def test_unknown_types_raise(self):
        cast = ansible_mitogen.utils.unsafe.cast
        self.assertRaises(TypeError, cast, set())
        self.assertRaises(TypeError, cast, Set())
        self.assertRaises(TypeError, cast, 4j)



from __future__ import absolute_import, division, print_function
__metaclass__ = type

import ansible
import ansible.utils.unsafe_proxy

import ansible_mitogen.utils

import mitogen
import mitogen.core
import mitogen.utils

__all__ = [
    'cast',
]

def _cast_to_dict(obj): return {cast(k): cast(v) for k, v in obj.items()}
def _cast_to_list(obj): return [cast(v) for v in obj]
def _cast_unsafe(obj): return obj._strip_unsafe()
def _passthrough(obj): return obj


# A dispatch table to cast objects based on their exact type.
# This is an optimisation, reliable fallbacks are required (e.g. isinstance())
_CAST_DISPATCH = {
    bytes: bytes,
    dict: _cast_to_dict,
    list: _cast_to_list,
    tuple: _cast_to_list,
    mitogen.core.UnicodeType: mitogen.core.UnicodeType,
}
_CAST_DISPATCH.update({t: _passthrough for t in mitogen.utils.PASSTHROUGH})

if hasattr(ansible.utils.unsafe_proxy.AnsibleUnsafeText, '_strip_unsafe'):
    _CAST_DISPATCH.update({
        ansible.utils.unsafe_proxy.AnsibleUnsafeBytes: _cast_unsafe,
        ansible.utils.unsafe_proxy.AnsibleUnsafeText: _cast_unsafe,
        ansible.utils.unsafe_proxy.NativeJinjaUnsafeText: _cast_unsafe,
    })
elif ansible_mitogen.utils.ansible_version[:2] <= (2, 16):
    _CAST_DISPATCH.update({
        ansible.utils.unsafe_proxy.AnsibleUnsafeBytes: bytes,
        ansible.utils.unsafe_proxy.AnsibleUnsafeText: mitogen.core.UnicodeType,
    })
else:
    mitogen_ver = '.'.join(str(v) for v in mitogen.__version__)
    raise ImportError("Mitogen %s can't unwrap Ansible %s AnsibleUnsafe objects"
                      % (mitogen_ver, ansible.__version__))


def cast(obj):
    """
    Return obj (or a copy) with subtypes of builtins cast to their supertype.

    This is an enhanced version of :func:`mitogen.utils.cast`. In addition it
    handles ``ansible.utils.unsafe_proxy.AnsibleUnsafeText`` and variants.

    There are types handled by :func:`ansible.utils.unsafe_proxy.wrap_var()`
    that this function currently does not handle (e.g. `set()`), or preserve
    preserve (e.g. `tuple()`). Future enhancements may change this.

    :param obj:
        Object to undecorate.
    :returns:
        Undecorated object.
    """
    # Fast path: obj is a known type, dispatch directly
    try:
        unwrapper = _CAST_DISPATCH[type(obj)]
    except KeyError:
        pass
    else:
        return unwrapper(obj)

    # Slow path: obj is some unknown subclass
    if isinstance(obj, dict): return _cast_to_dict(obj)
    if isinstance(obj, (list, tuple)): return _cast_to_list(obj)

    return mitogen.utils.cast(obj)



from __future__ import absolute_import, division, print_function
__metaclass__ = type

import os
import sys

import ansible.module_utils.external1

from ansible.module_utils.externalpkg.extmod import path as epem_path

def main():
    pass



# Copyright 2019, David Wilson
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice,
# this list of conditions and the following disclaimer.
#
# 2. Redistributions in binary form must reproduce the above copyright notice,
# this list of conditions and the following disclaimer in the documentation
# and/or other materials provided with the distribution.
#
# 3. Neither the name of the copyright holder nor the names of its contributors
# may be used to endorse or promote products derived from this software without
# specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

from __future__ import absolute_import, division, print_function
__metaclass__ = type

import os.path
import sys

try:
    import ansible_mitogen
except ImportError:
    base_dir = os.path.dirname(__file__)
    sys.path.insert(0, os.path.abspath(os.path.join(base_dir, '../../..')))
    del base_dir

import ansible_mitogen.connection


class Connection(ansible_mitogen.connection.Connection):
    transport = 'docker'

    @property
    def docker_cmd(self):
        """
        Ansible 2.3 synchronize module wants to know how we run Docker.
        """
        return 'docker'



import os

import testlib


class ConstructorTest(testlib.RouterMixin, testlib.TestCase):
    def test_okay(self):
        docker_path = testlib.data_path('stubs/stub-docker.py')
        context = self.router.docker(
            container='container_name',
            docker_path=docker_path,
        )
        stream = self.router.stream_by_id(context.context_id)

        argv = eval(context.call(os.getenv, 'ORIGINAL_ARGV'))
        self.assertEqual(argv[0], docker_path)
        self.assertEqual(argv[1], 'exec')
        self.assertEqual(argv[2], '--interactive')
        self.assertEqual(argv[3], 'container_name')
        self.assertEqual(argv[4], stream.conn.options.python_path)



#!/usr/bin/python

# Test functionality of ansible_mitogen.runner.PREHISTORIC_HACK_RE, which
# removes `reload(sys); sys.setdefaultencoding(...)` from an Ansible module
# as it is sent to a target. There are probably very few modules in the wild
# that still do this, reload() is a Python 2.x builtin function.
# issue #555: I'm a module that cutpastes an old hack.

from ansible.module_utils.basic import AnsibleModule

import sys
reload(sys)
sys.setdefaultencoding('utf8')


def main():
    module = AnsibleModule(argument_spec={})
    module.exit_json(ok=True)

if __name__ == '__main__':
    main()



import signal

import testlib
try:
    from unittest import mock
except ImportError:
    import mock

import mitogen.parent


class ReaperTest(testlib.TestCase):
    def test_calc_delay(self):
        broker = mock.Mock()
        proc = mock.Mock()
        proc.poll.return_value = None
        reaper = mitogen.parent.Reaper(broker, proc, True, True)
        self.assertEqual(50, int(1000 * reaper._calc_delay(0)))
        self.assertEqual(86, int(1000 * reaper._calc_delay(1)))
        self.assertEqual(147, int(1000 * reaper._calc_delay(2)))
        self.assertEqual(254, int(1000 * reaper._calc_delay(3)))
        self.assertEqual(437, int(1000 * reaper._calc_delay(4)))
        self.assertEqual(752, int(1000 * reaper._calc_delay(5)))
        self.assertEqual(1294, int(1000 * reaper._calc_delay(6)))

    def test_reap_calls(self):
        broker = mock.Mock()
        proc = mock.Mock()
        proc.poll.return_value = None

        reaper = mitogen.parent.Reaper(broker, proc, True, True)

        reaper.reap()
        self.assertEqual(0, proc.send_signal.call_count)

        reaper.reap()
        self.assertEqual(1, proc.send_signal.call_count)

        reaper.reap()
        reaper.reap()
        reaper.reap()
        self.assertEqual(1, proc.send_signal.call_count)

        reaper.reap()
        self.assertEqual(2, proc.send_signal.call_count)

        self.assertEqual(proc.send_signal.mock_calls, [
            mock.call(signal.SIGTERM),
            mock.call(signal.SIGKILL),
        ])
